@@ -81,7 +81,7 @@ typedef char bool;
 #define SM_TTY			64
 
 #define MAX_NUM_BACKENDS 128
-#define MAX_CONNECTION_SLOTS 128
+#define MAX_CONNECTION_SLOTS MAX_NUM_BACKENDS
 #define MAX_DB_HOST_NAMELEN	 128
 #define MAX_PATH_LENGTH 256
 
@@ -107,6 +107,7 @@ typedef struct {
 	double backend_weight;	/* normalized backend load balance ratio */
 	double unnormalized_weight; /* descripted parameter */
 	char backend_data_directory[MAX_PATH_LENGTH];
+	unsigned long long int standby_delay;		/* The replication delay against the primary */
 } BackendInfo;
 
 typedef struct {
@@ -127,9 +128,9 @@ typedef struct {
 	int			counter; /* used counter */
 	time_t 		create_time; /* connection creation time */
 	int load_balancing_node; /* load balancing node */
+	bool		connected;	/* true if frontend connected */
 } ConnectionInfo;
 
-
 /*
  * process information
  * This object put on shared memory.
@@ -254,11 +255,11 @@ typedef enum UserAuth
 	uaReject,
 	/*  uaKrb4, */
 	/*  uaKrb5, */
-	uaTrust
+	uaTrust,
 	/*  uaIdent, */
 	/*  uaPassword, */
 	/*  uaCrypt, */
-	/*  uaMD5 */
+	uaMD5
 #ifdef USE_PAM
 	,uaPAM
 #endif /* USE_PAM */
